#include <sys/sysinfo.h>
#include <machine/hal_sysinfo.h>
#include <machine/cpuconf.h>

int main(int argc,char *argv[])
{
        
        int     status=0;
        char    buf[40];
        int     st=0;
	
	status = getsysinfo(GSI_PLATFORM_NAME, buf, sizeof(buf), st, NULL,NULL);
	printf("\tPlatform = %s \n",buf);

}
