#include <sys/sysinfo.h>
#include <machine/hal_sysinfo.h>
#include <machine/cpuconf.h>

int main()
{
       	int status=0;
	char buf[20];
        int st=0;
	
	status = getsysinfo(GSI_PHYSMEM, buf, sizeof(buf), st, NULL, NULL);
	printf("\tPhysical Memory = %d \n",buf);
}
