#!/usr/bin/perl -w
#!/usr/bin/perl -d -w
#
# test.pl - Test harness for common.pm
# 
# Synopsis:
# 
#	test.pl [ -d ]
# 

use Getopt::Std;
use FileHandle;
use Sys::Hostname;
use common;

#######################################################################
# Global Variables
#######################################################################

# usually set by command line args
$Debug = 1;


#######################################################################
# Subroutines
#######################################################################

sub ProcessCommandLine
{
	if ( getopts('d') ) {
		if ( $opt_d ) { $Debug = 1; }
	}
	else {
		die "There were problems processing the command line.";
	}

	return;
}


sub TestSomething
{
	# $Debug = 0;
	# LoadConfigurationFile(1);

	# $Debug = 1;
	# LoadAndDumpProfile("allosaur.saic.com");
	# LoadAndDumpProfile("troodon.saic.com");

	# $Debug = 1;
	# UpdateListFile("OSList", "Solaris", $DeferFileUpdate);
	# UpdateListFile("OSList", "HP-UX", $DeferFileUpdate);
	# UpdateListFile("OSList", "Uglix", $DeferFileUpdate);
	# WriteListFile("OSList");

	# $Debug = 1;
	# CreateConfigurationFile();

	$Debug = 1;
	LoadConfigurationFile(1);

	# my $PathToSyssummDirectory = "$DocrootDirectory/$SyssummDirectory";
	# print "Path to syssumm dir: $PathToSyssummDirectory...\n";

	# GenerateWebPage("allosaur");

	return;
}


sub LoadAndDumpProfile
{	my $SystemName = shift;

	LoadProfile($SystemName);

	print "\nCategories:\n";
	foreach $Category (@ListOfCategories) {
		print "\t$Category\n";
	}

	print "\nValues for General categories:\n";
	my $Category = "General";
	foreach $Key (sort keys %{$Category}) {
		my $Value = ${$Category}{$Key};
		print "\tKey is $Key, Value is $Value\n";
	}

	return;
}


#######################################################################
# Mainline
#######################################################################

ProcessCommandLine();
TestSomething();

#######################################################################
# We're done!
#######################################################################
exit(0);
