package syssumm-site;

# syssumm-site.pm - these are site specific routines
#
# Rules for these subroutines:
# 
#	1.  These subroutines can be called in any order except
#	    for InitializeSite.  That will always be called
#	    before any others.
#	2.  These subroutines can be called in any order...
# 
# ==============================================================
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 
# 2 of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be 
# useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
# 
# Original source:
#	Copyright (c) 1999, 2000 SAIC (Bruce W. Mohler)
# 
# Contributed source:
#	Copyright (c) 1999, 2000 <author)
# 
# ==============================================================
# 
# $Header$
# 
# $Revision$
#
# $Log$

use common;

require Exporter;
@ISA = qw(Exporter);
###########################################################################
# the following sub-routines (&) are variables ($) will be 
# visible in a program using this module...
@EXPORT = qw(
	&InitializeSite
);
# 
# Note that you do not need to export the Return* subroutine
# names since they are being called by reference.
# 
###########################################################################

###########################################################################
# Public Global Variables (must be declared above)
###########################################################################


###########################################################################
# Private Global Variables
###########################################################################


###########################################################################
# Subroutines
###########################################################################

# 
# This subroutine is called before any others.  It's
# reason for existence is to plug in the references to
# subroutines below to actually generate the information.
# 

sub InitializeSite()
{
	print "InitializeSite()...\n" if $main::Debug;

	# plug in appropriate subroutines
# 	$main::Routines{Site}{XXX} = \&ReturnXXX;

	return;
}


###########################################################################
# Site
###########################################################################

sub ReturnXXX
{
	print "ReturnXXX()...\n" if $main::Debug;

	# XXX - do something!

	return "something";
}


###########################################################################
# End of syssumm-site.pm
###########################################################################

1;
